declare global {
  interface Window {
    cfdProxyUsage: { [key: string]: boolean };
    _RSS_PROPS_NOPROXY_: Record<any, any>;
  }
}

function getProxyProfile(profile: Record<any, any>) {
  window.cfdProxyUsage = window.cfdProxyUsage || {};

  function buildHandler(parentPath = "") {
    return {
      get(target: Record<any, any>, prop: string, receiver: any): any {
        if (prop in target) {
          // Don't log accesses to object or function types (Custom fields can't be those)
          // For objects, we need to create a new Proxy for the child object
          if (typeof target[prop] === "object") {
            return new Proxy(
              target[prop],
              buildHandler(`${parentPath}${prop}.`)
            );
          } else if (typeof target[prop] === "function") {
            return target[prop];
          } else {
            window.cfdProxyUsage[`${parentPath}${prop}`] = true;
          }
        }

        return target[prop];
      },
    };
  }

  return new Proxy(profile, buildHandler());
}

export { getProxyProfile };
