function makeID(length: number) {
  const characters =
    "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
  let result = "";
  let charactersLength = characters.length;
  for (let i = 0; i < length; i++) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return result;
}

function makeNumber(length: number) {
  const characters = "123456789";
  let result = "";
  let charactersLength = characters.length;
  for (let i = 0; i < length; i++) {
    result += characters.charAt(Math.floor(Math.random() * charactersLength));
  }
  return Number(result);
}

function randomizeHours() {
  return {
    sunday: {
      isClosed: false,
      openIntervals: [{ start: "1:01", end: "1:02" }],
    },
    monday: {
      isClosed: false,
      openIntervals: [{ start: "1:03", end: "1:04" }],
    },
    tuesday: {
      isClosed: false,
      openIntervals: [{ start: "1:05", end: "1:06" }],
    },
    wednesday: {
      isClosed: false,
      openIntervals: [{ start: "1:07", end: "1:08" }],
    },
    thursday: {
      isClosed: false,
      openIntervals: [{ start: "1:09", end: "1:10" }],
    },
    friday: {
      isClosed: false,
      openIntervals: [{ start: "1:11", end: "1:12" }],
    },
    saturday: {
      isClosed: false,
      openIntervals: [{ start: "1:13", end: "1:14" }],
    },
  };
}

function randomize(v: Record<any, any>): any {
  if (typeof v === "string") return makeID(12);
  if (typeof v === "number") return makeNumber(12);
  if (Array.isArray(v)) return v.map(randomize);
  if (typeof v === "object")
    return Object.keys(v).reduce(
      (out, key) => ({
        ...out,
        [key]: key === "hours" ? randomizeHours() : randomize(v[key]),
      }),
      {}
    );
}

// Returns a flattened object while preserving nested key names
//  using '.' as a separator ex.
//  {parent: {child: 'hello'}} => {parent.child: 'hello'}
function flattenObj(input: Record<any, any>, parentKey: string = "") {
  let flattened: Record<string, any> = {};
  Object.entries(input).forEach(([key, val]) => {
    if (typeof val === "object") {
      flattened = {
        ...flattened,
        ...flattenObj(val, `${parentKey}${key}.`),
      };
    } else {
      flattened[`${parentKey}${key}`] = val;
    }
  });
  return flattened;
}

// Returns a CSS selector using :nth-child to uniquely identify any DOM node
function getDOMNodeAddress(
  node: Node,
  addressIndices: Array<number> = []
): string {
  // Exit condition: This node has no parents, so we are at the bottom of the tree
  if (!node.parentNode) {
    // Convert the array of indices to a CSS selector first
    let selector = "html";
    // element at index 0 is always the `<html>` element, so shift() to ignore it
    addressIndices.shift();
    addressIndices.forEach((index) => {
      // TextNodes will have a -1 at the end since a textNode is not considered a child
      // :nth-child() is 1 indexed instead of 0 indexed
      if (index != -1) {
        selector = `${selector} > :nth-child(${index + 1})`;
      }
    });
    return selector;
  }

  // Find the index of this node among it's siblings and recurse
  if (node instanceof HTMLElement) {
    const siblingIndex = Array.from(node.parentNode.children).indexOf(node);
    return getDOMNodeAddress(node.parentNode, [
      siblingIndex,
      ...addressIndices,
    ]);
  } else {
    return getDOMNodeAddress(node.parentNode, [-1, ...addressIndices]);
  }
}

export {
  makeID,
  makeNumber,
  randomizeHours,
  randomize,
  flattenObj,
  getDOMNodeAddress,
};
